/*
  -------------------- file region.h
  Created 19 Jun 94 by Andrew Shapira, Rensselaer Polytechnic Institute.

	Copyright (C) 1994 Andrew Shapira.

	This file is part of the Ntuplib library.

	Ntuplib is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Ntuplib is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with the Ntuplib Library; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _190694_153849
#define _190694_153849

#include "../libinclude/ntup.h"
#include "../general/teval.h"

struct to_region_struct {
  int p,q;
  int n;
  int tuple_kind;
  const struct coord_struct *pi;
  double max_slice_nodes; /* max nodes to examine during this slice */
  int width;              /* search width */
  int found_first;        /* has a solution been found yet for given p,q? */
  double p_nodes;         /* nodes examined prior to this region, this p,q */
  int nactive_regions;    /* number of regions */
  int update_interval;
  FILE *f;
  int back_print;
  unsigned ioflags;
  double *lev_nodes;
};

/* searched for a while, did not exhaust region, did not find solution */
#define SEARCH_INCONCLUSIVE   0
/* completed search of entire region, did not find solution */
#define SEARCH_EXHAUSTED      1
/* found first solution for given p,q values */
#define SEARCH_FOUND_FIRST    2


struct from_region_struct {
  int status;                     /* SEARCH_XX constant */
  double nodes;                   /* number of new nodes inspected */
  double solutions;               /* number of solutions we found */
};

/* These structures are structure-assigned, so be careful about
   putting too much information here.  Also be careful about putting
   any allocated fields here.
*/
struct region_struct {
  /* region id; used for identifying region to user */
  int id;

  int *ps;          /* piidc's of partial solution components */
  int base_len;     /* length of base part of partial solution */
  int extend_len;   /* length of extended part of partial solution */

  /* Flag used to mark the special situation when the last node
     in the region is a solution.
  */
  int nomore;
  /*  Random seed used to help give different results for
      different runs.  Unsigned for consistency with vec_u_hash.
  */
  unsigned seed;
  struct teval_struct teval[1];   /* does not really belong here */
};

void back2_search_region P(( const struct to_region_struct * const,
  struct region_struct * const,
  struct from_region_struct * const, struct pix_struct * const ));


#endif /* _190694_153849 */
