/*
  -------------------- file vector.h
  Created Spring 92 by Andrew Shapira, Rensselaer Polytechnic Institute.

	Copyright (C) 1992 Andrew Shapira.

	This file is part of the Ntuplib library.

	Ntuplib is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Ntuplib is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with the Ntuplib Library; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*  Note: Contains the prototypical way to handle inline functions.  */

#ifndef _020992_124154_vector
#define _020992_124154_vector

#include "common.h"


/*  The naming convention for vector routines: without a specifier, the
    vector type is int.  This is the same convention that is used for matrices.
*/

int vec_cmp_distrib P(( const int *, const int *,
  const int, const int, const int, int * ));
void vec_print P((FILE *, const int *, const int));
void vec_print_offset P(( FILE *, const int *, const int, const int ));
void vec_u_print P((FILE *, const unsigned int *, const int));
void vec_uchar_print P((FILE *, const unsigned char *, const int));
int vec_print2 P((FILE *, const int *, const int, const int, const int));
int vec_u_print2 P((FILE *, const unsigned int *,const int,const int,const int));
int vec_uchar_print2 P((FILE *, const unsigned char *,const int,const int,const int));
void vec_print_as_matrix
    P(( FILE *, const int *, const int, const unsigned char *, const unsigned char * ));
void vec_u_print_as_matrix
    P(( FILE *, const unsigned char *, const int, const int *, const int * ));
int vec_u_read P((FILE *,unsigned char *, const int));
int vec_read P((FILE *,int *, const int));
void vec_to_table
  P(( const int *, const int, const int, const int, int * ));
void vec_table_to_vec
  P(( const int *, const int, const int, const int, int * ));
unsigned vec_u_hash P(( const unsigned * const , const int ));
unsigned vec_uchar_hash P(( const unsigned char * const, const int ));
unsigned int vec_hash P(( const int * const, const int ));

/**************************************************************************
  inline functions (be careful about changing the order of func. definition)
**************************************************************************/

/*  declarations (included for compatibility with non-gnuc compilers) */

#ifndef __GNUC__
int * vec_alloc P(( const int, void *(*) P((const int)) ));
unsigned int * vec_u_alloc P(( const int, void  *(*) P((const int)) ));
unsigned char * vec_uchar_alloc P(( const int, void *(*) P((const int)) ));
double * vec_d_alloc P(( const int, void  *(*) P((const int)) ));
void vec_free P(( int *, void (*) P((void *)) ));
void vec_u_free P(( unsigned int *, void  (*) P((void *)) ));
void vec_uchar_free P(( unsigned char *, void (*) P((void *)) ));
int vec_cmp P(( const int *, const int *, const int ));
int vec_u_cmp P(( const unsigned *, const unsigned *, const int ));
int vec_uchar_cmp P(( const unsigned char *,const unsigned char *,const int ));
int vec_cmp_scalar P(( const int *, const int, const int ));
void vec_copy P((int *, const int *, const int ));
void vec_u_copy P(( unsigned int *, const unsigned int *, const int ));
void vec_uchar_copy P(( unsigned char *, const unsigned char *, const int ));
void vec_zero P(( int *, const int ));
void vec_u_zero P(( unsigned int *, const int ));
void vec_uchar_zero P(( unsigned char *, const int ));
void vec_d_zero P(( double *, const int ));
void vec_minmax P(( const int *, const int, int *, int * ));
void vec_u_minmax P(( const unsigned int *, const int,
  unsigned int *, unsigned int * ));
void vec_uchar_minmax P(( const unsigned char *, const int,
  unsigned char *, unsigned char * ));
void vec_set P(( int *, const int, const int ));
void vec_d_copy P(( double *, const double *, const int ));
int vec_count_scalar P(( const int *, const int, const int ));
int vec_count_scalar_not P(( const int *, const int, const int ));
void vec_population_count P(( const int *, const int, int * ));
int vec_sum P(( const int * const, const int ));
int vec_uchar_sum P(( const unsigned char * const, const int ));
int vec_is_0 P(( const int *, const int ));
int vec_find_scalar P(( const int *, const int, const int ));
int vec_find_scalar_not P(( const int *, const int, const int ));
void vec_apply_map P(( int *,const int,const int,const int *,const int * ));
void vec_u_apply_map P (( unsigned int *, const int,
  const int, const int *, const unsigned int * ));
void vec_add_scalar P(( int *, const int, const int ));
#endif  /*__GNUC__*/

/*  note: inline is defined to the empty string unless we're using gnuc */
#undef  INLINE
#ifdef  SRCFILE_LIBPROG_VECTOR
#define INLINE  __inline__
#else
#define INLINE  __inline__ extern
#endif

#if defined(__GNUC__) || defined(SRCFILE_LIBPROG_VECTOR)

/*  allocate a length n int vector.
*/
INLINE int *
vec_alloc(
const int n,
void  *(*allocfunc) P((const int))
)
{
  return((*allocfunc)(n*sizeof(int)));
}

INLINE unsigned int *
vec_u_alloc(
const int n,
void  *(*allocfunc) P((const int))
)
{
  return((*allocfunc)(n*sizeof(unsigned int)));
}

INLINE unsigned char *
vec_uchar_alloc(
const int n,
void  *(*allocfunc) P((const int))
)
{
  return((*allocfunc)(n*sizeof(unsigned char)));
}

INLINE double *
vec_d_alloc(
const int n,
void  *(*allocfunc) P((const int))
)
{
  return((*allocfunc)(n*sizeof(double)));
}

INLINE void
vec_free(
int   *v,
void  (*freefunc) P((void *))
)
{
  (*freefunc) (v);
}

INLINE void
vec_u_free(
unsigned int    *v,
void  (*freefunc) P((void *))
)
{
  (*freefunc) (v);
}

INLINE void
vec_uchar_free(
unsigned char   *v,
void  (*freefunc) P((void *))
)
{
  (*freefunc) (v);
}


/*  Compare vectors v1 and v2 of length n.  Returns -1, 0, or 1 according
    to whether v1 is lexicographically smaller than, equal to, or greater
    than v1.  Vector length is allowed to be 0.
*/
INLINE int
vec_cmp(
const int   *v1,
const int   *v2,
const int   n
)
{
  int   i;

  for (i = 0; i < n; i++) {
    if (v1[i] != v2[i])
      return(v1[i] - v2[i]);
  }
  return(0);
}

INLINE int
vec_u_cmp(
const unsigned   *v1,
const unsigned   *v2,
const int   n
)
{
  int   i;

  for (i = 0; i < n; i++) {
    if (v1[i] != v2[i])
      return(v1[i] - v2[i]);
  }
  return(0);
}

INLINE int
vec_uchar_cmp(
const unsigned char   *v1,
const unsigned char   *v2,
const int   n
)
{
  int   i;

  for (i = 0; i < n; i++) {
    if (v1[i] != v2[i])
      return(v1[i] - v2[i]);
  }
  return(0);
}

/*  Compare a vector and see if all of its elements are a certain scalar "val".
    Returns -1, 0, or 1 according to whether the vector is lexicographically
    smaller than, equal to, or greater than val.
*/
INLINE int
vec_cmp_scalar(
const int   *vec,
const int   n,
const int   val
)
{
  int   i;

  for (i = 0; i < n; i++) {
    if (vec[i] != val)
      return (vec[i] - val);
  }
  return(0);
}

/*  Copy n elements of integer vector vsource to vdest.
*/
INLINE void
vec_copy(
int         *vdest,
const int   *vsource,
const int   n
)
{
  int   i;

  for (i = 0; i < n; i++)
    vdest[i] = vsource[i];
}

INLINE void
vec_u_copy(
unsigned int        *vdest,
const unsigned int  *vsource,
const int           n
)
{
  int   i;

  for (i = 0; i < n; i++)
    vdest[i] = vsource[i];
}

INLINE void
vec_uchar_copy(
unsigned char         *vdest,
const unsigned char   *vsource,
const int             n
)
{
  (void) memcpy(vdest, vsource, n);
}

INLINE void
vec_d_copy(
double        *vdest,
const double  *vsource,
const int     n
)
{
  int   i;

  for (i = 0; i < n; i++)
    vdest[i] = vsource[i];
}

/*  Zero the first n elements of integer vector v.
*/
INLINE void
vec_zero(
int   *v,
const int   n
)
{
  int   i;

  for (i = 0; i < n; i++)
    v[i] = 0;
}

INLINE void
vec_u_zero(
unsigned int   *v,
const int       n
)
{
  int   i;

  for (i = 0; i < n; i++)
    v[i] = 0;
}

INLINE void
vec_uchar_zero(
unsigned char   *v,
const int         n
)
{
  memset(v, 0, n);
}

INLINE void
vec_d_zero(
double      *v,
const int   n
)
{
  int   i;

  for (i = 0; i < n; i++)
    v[i] = 0;
}

/*  Find the smallest and largest values in an INT vector. */
INLINE void
vec_minmax(
const int   *v,
const int   n,
int         *minret,    /* NULL == do not return value */
int         *maxret
)
{
  int   col,min,max;

  min = max = v[0];
  for (col = 0; col < n; col++) {
    max = MAX(max, v[col]);
    min = MIN(min, v[col]);
  }
  if (minret)
    *minret = min;
  if (maxret)
    *maxret = max;
}

INLINE void
vec_u_minmax(
const unsigned int  *v,
const int           n,
unsigned int        *minret,
unsigned int        *maxret
)
{
  int   col;
  unsigned int min,max;

  min = max = v[0];
  for (col = 0; col < n; col++) {
    max = MAX(max, v[col]);
    min = MIN(min, v[col]);
  }
  if (minret)
    *minret = min;
  if (maxret)
    *maxret = max;
}

INLINE void
vec_uchar_minmax(
const unsigned char   *v,
const int             n,
unsigned char         *minret,    /* NULL == do not return value */
unsigned char         *maxret
)
{
  int   col;
  unsigned char min,max;

  min = max = v[0];
  for (col = 0; col < n; col++) {
    max = MAX(max, v[col]);
    min = MIN(min, v[col]);
  }
  if (minret)
    *minret = min;
  if (maxret)
    *maxret = max;
}

/*  Set the first n elements of integer vector v to "val".
    Note: use "memset" for byte vectors.
*/
INLINE void
vec_set(
int   *v,
const int   n,
const int   val
)
{
  int   i;

  for (i = 0; i < n; i++)
    v[i] = val;
}

/*  Count the number of elements that are a specific value.
*/
INLINE int
vec_count_scalar(
const int *v,
const int n,
const int val
)
{
  int   i,count;

  for (i = 0, count = 0; i < n; i++)
    count += (v[i] == val);
  return(count);
}

/*  Count the number of elements that are not a specific value.
*/
INLINE int
vec_count_scalar_not(
const int *v,
const int n,
const int val
)
{
  int   i,count;

  for (i = 0, count = 0; i < n; i++)
    count += (v[i] != val);
  return(count);
}

/*  Determine population count of v.  (How many times each value appears.)
    Assumes values of *v are all in the range 0 .. n-1.
    Returns population in count[0] .. count[n-1].
*/
INLINE void
vec_population_count(
const int   *v,
const int   n,
int         *count
)
{
  int   i;

  vec_zero(count,n);
  for (i = 0; i < n; i++)
    count[v[i]]++;
}

/*  Sum the entries in a vector.
*/
INLINE int
vec_sum(
const int *const v,
const int n
)
{
  int   count, i;

  count = 0;
  for (i = 0; i < n; i++)
    count += v[i];
  return(count);
}

/*  Sum the entries in a vector.
*/
INLINE int
vec_uchar_sum(
const unsigned char *const v,
const int n
)
{
  int   count, i;

  count = 0;
  for (i = 0; i < n; i++)
    count += v[i];
  return(count);
}

INLINE int
vec_is_0(
const int       *v,
const int       n
)
{
  int   i;

  for (i = 0; i < n; i++) {
    if (v[i])
      return(0);
  }
  return(1);
}

/*  Find the position of the first occurrence of scalar "s" in vector "v".
    Returns 0..n-1 if found, -1 if not found.
*/
INLINE int
vec_find_scalar(
const int   *v,
const int   n,        /* is allowed to be 0 or negative (causes return(-1)) */
const int   s
)
{
  int   i;

  for (i = 0; i < n; i++) {
    if (v[i] == s)
      return(i);
  }
  return(-1);
}

/*  Find the position of the first element of v that is not s.
    Returns 0..n-1 if found, -1 if not found.
*/
INLINE int
vec_find_scalar_not(
const int   *v,
const int   n,
const int   s
)
{
  int   i;

  for (i = 0; i < n; i++) {
    if (v[i] != s)
      return(i);
  }
  return(-1);
}

/*  Apply a mapping of values to a vector.  That is,
    dest[i] = map[source[i]].  It is allowable to have source == dest.
*/
INLINE void
vec_apply_map(
int         *dest,
const int   n,
const int   min,  /* min. value occuring in source */
const int   *map, /* map[0] == what to map min to; map[1] map for min+1, etc. */
const int   *source
)
{
  int   i;

  for (i = 0; i < n; i++) {
    dest[i] = map[source[i]-min];
  }
}

INLINE void
vec_u_apply_map(
unsigned int          *dest,
const int             n,
const int             min,
const int             *map,
const unsigned int    *source
)
{
  int   i;

  for (i = 0; i < n; i++) {
    dest[i] = map[source[i]-min];
  }
}

INLINE void
vec_add_scalar(
int         *v,
const int   n,
const int   add
)
{
  int   i;

  for (i = 0; i < n; i++)
    v[i] += add;
}


#endif  /* defined(__GNUC__) || defined(source file) */

#endif /* _020992_124154_vector */
